part of katana_form;

/// Create buttons for forms.
///
/// Specify the label to be displayed on the button in [label].
///
/// The [onPressed] allows you to describe the process when the button is pressed.
///
/// If [icon] is specified, an icon is displayed to the left of the label.
///
/// If [enabled] is set to `false`, the button cannot be pressed.
///
/// The style of the button can be changed by specifying [style].
///
/// フォーム用のボタンを作成します。
///
/// [label]にボタンに表示するラベルを指定します。
///
/// [onPressed]でボタンが押された場合の処理を記述できます。
///
/// [icon]を指定した場合は、ラベルの左側にアイコンが表示されます。
///
/// [enabled]を`false`にした場合ボタンが押せなくなります。
///
/// [style]を指定するとボタンのスタイルを変更することが可能です。
class FormButton extends StatelessWidget {
  /// Create buttons for forms.
  ///
  /// Specify the label to be displayed on the button in [label].
  ///
  /// The [onPressed] allows you to describe the process when the button is pressed.
  ///
  /// If [icon] is specified, an icon is displayed to the left of the label.
  ///
  /// If [enabled] is set to `false`, the button cannot be pressed.
  ///
  /// The style of the button can be changed by specifying [style].
  ///
  /// フォーム用のボタンを作成します。
  ///
  /// [label]にボタンに表示するラベルを指定します。
  ///
  /// [onPressed]でボタンが押された場合の処理を記述できます。
  ///
  /// [icon]を指定した場合は、ラベルの左側にアイコンが表示されます。
  ///
  /// [enabled]を`false`にした場合ボタンが押せなくなります。
  ///
  /// [style]を指定するとボタンのスタイルを変更することが可能です。
  const FormButton(
    this.label, {
    super.key,
    this.onPressed,
    this.enabled = true,
    this.icon,
    this.style,
  });

  /// Defines the style of the button.
  ///
  /// ボタンのスタイルを定義します。
  final FormStyle? style;

  /// Button labels.
  ///
  /// ボタンのラベル。
  final String label;

  /// Processing when a button is pressed.
  ///
  /// ボタンが押された場合の処理。
  final VoidCallback? onPressed;

  /// Button icon. It appears to the left of the label.
  ///
  /// ボタンのアイコン。ラベルの左側に表示されます。
  final IconData? icon;

  /// If `false`, the button is deactivated and cannot be pressed.
  ///
  /// `false`になった場合、ボタンが非有効化され押せなくなります。
  final bool enabled;

  @override
  Widget build(BuildContext context) {
    final borderWidth = style?.borderWidth ?? 0.0;
    final borderColor = style?.borderColor;
    final borderRadius = style?.borderRadius;
    final buttonStyle = TextButton.styleFrom(
      padding: style?.contentPadding,
      textStyle: style?.textStyle,
      foregroundColor: style?.color ?? Theme.of(context).colorScheme.onPrimary,
      disabledForegroundColor:
          style?.color ?? Theme.of(context).colorScheme.onPrimary,
      backgroundColor:
          style?.backgroundColor ?? Theme.of(context).colorScheme.primary,
      shape: RoundedRectangleBorder(
        side: borderWidth <= 0 || borderColor == Colors.transparent
            ? BorderSide.none
            : BorderSide(
                color: borderColor ?? Colors.black,
                width: borderWidth,
              ),
        borderRadius: borderRadius ?? BorderRadius.zero,
      ),
    ).addState(
      foregroundColor: style?.disabledColor,
      backgroundColor: style?.backgroundColor == Colors.transparent
          ? style?.backgroundColor
          : (style?.disabledBackgroundColor ?? Theme.of(context).disabledColor),
      state: {
        MaterialState.disabled,
      },
    );
    return Container(
      width: style?.width ?? double.infinity,
      margin: style?.padding,
      child: icon != null
          ? TextButton.icon(
              style: buttonStyle,
              icon: Icon(
                icon,
              ),
              label: Text(label, style: style?.textStyle),
              onPressed: enabled ? onPressed : null,
            )
          : TextButton(
              style: buttonStyle,
              child: Text(label, style: style?.textStyle),
              onPressed: enabled ? onPressed : null,
            ),
    );
  }
}

extension _ButtonStyleExtension on ButtonStyle {
  ButtonStyle addState({
    Color? backgroundColor,
    Color? foregroundColor,
    Set<MaterialState> state = const {
      MaterialState.focused,
      MaterialState.hovered,
      MaterialState.pressed,
      MaterialState.selected,
    },
  }) {
    return copyWith(
      backgroundColor: MaterialStateProperty.resolveWith((st) {
        if (st.containsAny(state)) {
          return backgroundColor ?? this.backgroundColor?.resolve(st);
        }
        return this.backgroundColor?.resolve(st);
      }),
      foregroundColor: MaterialStateProperty.resolveWith((st) {
        if (st.containsAny(state)) {
          return foregroundColor ?? this.foregroundColor?.resolve(st);
        }
        return this.foregroundColor?.resolve(st);
      }),
    );
  }
}
